% System aberration calibration using gradient descent optimation on intensity
% Scanimage and DM required.

clear ZVector corDMcommand;
hAOROIctrl.hDMctrl.triggeron;  % enable DM trigger mode
hAOROIctrl.hDMctrl.asynchronous; % enable DM trigger mode
load('Z2C.mat'); % load Z2C matrix of the DM
imgsize = 300; % Set image size
zernikemodeN=60; % Set zenrnike mode number
Zstart = 4; % zernike mode started
Zend = 60; % zernike mode ended
Ncycle = 25; % iterative total cycles
d=0.01; % factor
Nrep=8; % number of times to repeat measurment cycle, for averaging purposes.
Zdescent=0.02; %step factor
zernike_opt_history=zeros(Ncycle,zernikemodeN);
zernike_current=zeros(1,zernikemodeN);
indexzcommand = zeros(1,zernikemodeN);
indexzcommand(Zstart:Zend) = 1;
% generate the zernike vector to be loaded, format: [0;+d1;-d1;+d2;-d2;....]
i=1;
j=1;
Vzero=zeros(1,zernikemodeN);
while i<=length(indexzcommand)
   if indexzcommand(i)==1
       Vzero(i)=1;
       Zdelta(j,:)=d*Vzero;
       Zdelta(j+1,:)=(-1)*d*Vzero;
       Vzero(i)=0;
       j=j+2;
   end
    i=i+1;
end
Framepercyc=size(Zdelta,1);
ZVector = zeros(size(Zdelta,1),size(Zdelta,2));

%% start the system aberration calibrating
h = waitbar(0,'System aberration optimization...');
for C = 1:Ncycle %interitive cycle times    
    for m = 1:2*(Zend-Zstart+1)
        ZVector(m,:)= Zdelta(m,:)+ zernike_current;
    end
    imgname = sprintf('%s%d','cycle',C);
    % setup ScanImage scanning parameter
    assert(strcmpi(hSI.acqState,'idle'));  
    hSI.hScan2D.logFilePath = '..\data\';      
    hSI.hScan2D.logFileStem = imgname ;     % set the base file name for the Tiff file
    hSI.hScan2D.logFileCounter = 1;         % set the current Tiff file number
    hSI.hChannels.loggingEnable = true;     % enable logging
    hSI.hStackManager.framesPerSlice = Framepercyc*Nrep;   % set number of frames to capture in one Grab
    
    DMcommand = (ZVector*Z2C);
    %send command to DM controlled by ScanImage frame trigger
    hAOROIctrl.hDMctrl.sendpatterndata(DMcommand);
    % Start grab images 
    hSI.startGrab();                   
    while ~strcmpi(hSI.acqState,'idle')
         pause(0.5)
    end   
    hAOROIctrl.hDMctrl.stopDM; 
     
    % Calculate the gradient based on the intensity changes
    filename=sprintf('%s%d%s','cycle',C,'_00001.tif');
    nFrames=size(ZVector,1)*Nrep;
    [img framenum] = imreadtifseries(filename,nFrames);
    imagesquence = int16(zeros(imgsize,imgsize,size(ZVector,1)));
    for N=1:Nrep
        imagesquence = imagesquence + img(:,:,(1+size(ZVector,1)*(N-1)):(size(ZVector,1)*N));
    end
    imagesquence=imagesquence./Nrep;
    for i=1:size(imagesquence,3)
       intensity(i)=mean(mean(imagesquence(:,:,i)));
    end
    gradient=(-intensity(2:2:end)+intensity(1:2:end-1))/(2*d);
    gradient=gradient/sqrt(sum(gradient.^2))*Zdescent;
    % update the current zernike mode
    zernike_current(Zstart:Zend) = zernike_current(Zstart:Zend) + gradient;
    zernike_opt_history(C,:) = zernike_current;
    
    waitbar(C/Ncycle);
end  
hAOROIctrl.hDMctrl.triggeroff;
close(h);

%% save the DM command for system aberration
figure,plot(zernike_opt_history(:,4:47));
figure;bar(zernike_current);
DMcommand_system_aberration = zernike_current*Z2C;
